#PMN Note, had to edit event.py

MODULES = {
        #"GoogleContactTwoWay"  : { "type": "dataprovider" },
        "GoogleCalendarTwoWay" : { "type": "dataprovider" },
        #"GoogleMemoTwoWay"     : { "type": "dataprovider" },
        #"GoogleTasksTwoWay"    : { "type": "dataprovider" },
}

import gtk
import gobject

import conduit
from conduit import log,logd,logw
import conduit.DataProvider as DataProvider
import conduit.Utils as Utils
import conduit.Exceptions as Exceptions

import conduit.datatypes.Contact as Contact
import conduit.datatypes.Event as Event
import conduit.datatypes.Note as Note

from datetime import datetime, date
import vobject

import dateutil.parser
from dateutil.tz import tzutc, tzlocal

import gdata.calendar.service
import gdata.service
import gdata.calendar
import atom

#TODO encapsulate in new class
#class GoogleEvent():
#    def SetUID(uid):
#    def GetUID(uid):
#    def SetTitle(title):
#    def GetTitle():
#    def SetLocation(location):
#    def GetLocation():
#    def SetDateStart(dateStart):
#    def GetDateStart():
#    def SetDateEnd(dateEnd):
#    def GetDateEnd():
#    def SetRecurrence(recurrence):
#    def GetRecurrence():
#    def GetICalString():
#    def ConvertFromICalString():
#
#    def DeleteFromCalendar()
#    def CreateInCalendar()

    
class GoogleCalendarTwoWay(DataProvider.TwoWay):
    _name_ = "Google Calendar"
    _description_ = "Sync your Calendar"
    _category_ = DataProvider.CATEGORY_OFFICE
    _module_type_ = "twoway"
    _in_type_ = "event"
    _out_type_ = "event"
    _icon_ = "contact-new"
    
    def __init__(self):
        DataProvider.TwoWay.__init__(self)
        log("Initializing my GoogleCalanderTwoWay")
        
        self.selectedCalendarName = "default"
        self.selectedCalendarURI = "default"
        self.username = ""
        self.password = ""
        self.calServ = gdata.calendar.service.CalendarService()
        self.events = dict()
        #self.need_configuration(True)
        #self.set_configured(False)
        
    def _loadCalendars(self, widget, tree):
        usernameEntry = tree.get_widget("username")
        passwordEntry = tree.get_widget("password")
        sourceComboBox = tree.get_widget("sourceComboBox")
        calendarLbl = tree.get_widget("calendarLbl")
        okBtn = tree.get_widget("okBtn")

        self.username = usernameEntry.get_text()
        self.password = passwordEntry.get_text()
              
        self.calServ.email = self.username
        self.calServ.password = self.password
        self.calServ.ProgrammaticLogin()        
        allCalendars = self.calServ.GetCalendarListFeed().entry

        store = sourceComboBox.get_model()
        store.clear()
        for calendar in allCalendars:
            calendarURI = calendar.id.text.split('/')[-1]
            calendarName = calendar.title.text
            rowref = store.append( (calendarName, calendarURI) )
            if calendarURI == self.selectedCalendarURI:
                sourceComboBox.set_active_iter(rowref)

        sourceComboBox.set_sensitive(True)
        calendarLbl.set_sensitive(True)
        okBtn.set_sensitive(True)
        
    def configure(self, window):
        tree = Utils.dataprovider_glade_get_widget(
                        __file__, 
                        "config.glade",
                        "GoogleCalendarConfigDialog"
                        )
        
        tree.get_widget("username").set_text(self.username)
        tree.get_widget("password").set_text(self.password)
            
        sourceComboBox = tree.get_widget("sourceComboBox")       
        store = gtk.ListStore(gobject.TYPE_STRING,gobject.TYPE_STRING)
        sourceComboBox.set_model(store)

        cell = gtk.CellRendererText()
        sourceComboBox.pack_start(cell, True)
        sourceComboBox.add_attribute(cell, 'text', 0)
        sourceComboBox.set_active(0)

        rowref = store.append( (self.selectedCalendarName, self.selectedCalendarURI) )
        sourceComboBox.set_active_iter(rowref)

        dlg = tree.get_widget("GoogleCalendarConfigDialog")
        dlg.set_transient_for(window)
        
        signalConnections = { "on_loadCalendarsBtn_clicked" : (self._loadCalendars, tree) }
        tree.signal_autoconnect( signalConnections )
        
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            self.selectedCalendarName = store.get_value(sourceComboBox.get_active_iter(), 0)
            self.selectedCalendarURI = store.get_value(sourceComboBox.get_active_iter(), 1)
        dlg.destroy()  
        
    def refresh(self):
        DataProvider.TwoWay.refresh(self)
        self.calServ.email = self.username
        self.calServ.password = self.password
        self.calServ.ProgrammaticLogin()
        calQuery = gdata.calendar.service.CalendarEventQuery(user = self.selectedCalendarURI)
        eventFeed = self.calServ.CalendarQuery(calQuery)
        self.events.clear()
        for event in eventFeed.entry:            
            eventUID = event.id.text.split('/')[-1] + "@google.com"
            self.events[eventUID] = event
        
    def finish(self):
        self.events.clear()
        
    def get_all(self):
        return self.events.keys()
        
    def get_num_items(self):
        DataProvider.TwoWay.get_num_items(self) 
        return len(self.events)

    def get(self, LUID):
        DataProvider.TwoWay.get(self, LUID)

        googleEvent = self.events[LUID]
        iCalEvent = vobject.iCalendar().add('vevent')
        conduitEvent = Event.Event(LUID)     
        logd(str(googleEvent))
             
        iCalEvent.add('uid').value = LUID
        conduitEvent.set_UID(LUID)
        
        if googleEvent.title.text != None:    
            iCalEvent.add('summary').value = googleEvent.title.text
        if googleEvent.content.text != None:
            iCalEvent.add('description').value = googleEvent.content.text
        if googleEvent.where[0].text != None:
            iCalEvent.add('location').value = googleEvent.where[0].text
        if googleEvent.published.text != None:
            iCalEvent.add('created').value = dateutil.parser.parse( googleEvent.published.text ).astimezone(tzutc())    
        if googleEvent.updated.text != None:
            mTimeUTC = dateutil.parser.parse( googleEvent.updated.text ).astimezone(tzutc())
            dateModified = iCalEvent.add('last-modified')
            dateModified.value = mTimeUTC
            mTimeLocal = mTimeUTC.astimezone(tzlocal())
            mTimeLocalWithoutTZ = mTimeLocal.replace(tzinfo=None)
            #mtimes need to be naive and local
            #Shouldn't Conduit use non-naive mTimes?
            conduitEvent.set_mtime( mTimeLocalWithoutTZ )

        #iCalEvent.vevent.add('dtstamp').value = 

        if len(googleEvent.when) == 0:
            recurVObject = vobject.readOne('BEGIN:VEVENT\r\n' + googleEvent.recurrence.text + '\r\nEND:VEVENT\r\n')
            iCalEvent.add('dtend')
            iCalEvent.dtend = recurVObject.dtend
            iCalEvent.add('dtstart')
            iCalEvent.dtstart = recurVObject.dtstart
            iCalEvent.add('rrule')
            iCalEvent.rrule = recurVObject.rrule
        else:
            dateStart = iCalEvent.add('dtstart')
            dateEnd = iCalEvent.add('dtend')
            eventTimes = googleEvent.when[0]

            startDateTime = dateutil.parser.parse( eventTimes.start_time )
            endDateTime = dateutil.parser.parse( eventTimes.end_time )
            #This doesn't seem to be the best way to check for all day events
            #i.e. timeless start and end date
            if (eventTimes.start_time.find('T') != -1) and (eventTimes.start_time.find('T') != -1):
                dateStart.value = startDateTime.astimezone(tzutc())
                dateEnd.value = endDateTime.astimezone(tzutc())
            else:
                dateStart.value = startDateTime.date()
                dateEnd.value = endDateTime.date() 

        logd(iCalEvent.serialize())
        conduitEvent.set_from_ical_string(iCalEvent.serialize())
        return conduitEvent
            
                   
    def _createEvent(self, conduitEvent):
        iCalEvent = vobject.readOne(conduitEvent.get_ical_string())
        googleEvent = gdata.calendar.CalendarEventEntry()
        
        if iCalEvent.contents.has_key('summary'):
            googleEvent.title = atom.Title(text=iCalEvent.summary.value)
        if iCalEvent.contents.has_key('description'):
            googleEvent.content = atom.Content(text=iCalEvent.description.value)
        if iCalEvent.contents.has_key('location'):
            googleEvent.where.append(gdata.calendar.Where(value_string=iCalEvent.location.value))
            
        if iCalEvent.contents.has_key('rrule'):
            recurrenceStr = iCalEvent.dtstart.serialize()
            recurrenceStr += iCalEvent.dtend.serialize() 
            recurrenceStr += iCalEvent.rrule.serialize()
            googleEvent.recurrence = gdata.calendar.Recurrence(text=recurrenceStr)
        else:
            eventTimes = gdata.calendar.When()
            startDateTime = dateutil.parser.parse(iCalEvent.dtstart.value)
            endDateTime = dateutil.parser.parse(iCalEvent.dtend.value)
            #This doesn't seem to be the best way to check for all day events
            #i.e. timeless start and end date
            if (iCalEvent.dtstart.value.find('T') != -1) and (iCalEvent.dtend.value.find('T') != -1):
                eventTimes.start_time = startDateTime.isoformat()
                eventTimes.end_time = endDateTime.isoformat()
            else:
                eventTimes.start_time = startDateTime.date().isoformat()
                eventTimes.end_time = endDateTime.date().isoformat()
            googleEvent.when.append(eventTimes)
            
        calendarFeed = '/calendar/feeds/' + self.selectedCalendarURI + '/private/full'
        newEvent = self.calServ.InsertEvent(googleEvent, calendarFeed)
        return newEvent.id.text.split('/')[-1] + "@google.com"
        
        
    def _deleteEvent(self, LUID):
        event = self.events[LUID]
        self.calServ.DeleteEvent(event.GetEditLink().href)
        
    def _updateEvent(self, LUID, conduitEvent):
        self._deleteEvent(LUID)
        uid = self._createEvent(conduitEvent)
        return uid

    def delete(self, LUID):
        self._deleteEvent(LUID)
        
    def put(self, obj, overwrite, LUID=None):
        #Following taken from EvolutionModule
        DataProvider.TwoWay.put(self, obj, overwrite, LUID)
        if LUID != None:
            existing = self.events.get(LUID, None)
            if existing != None:
                if overwrite == True:
                    uid = self._updateEvent(LUID, obj)
                    return uid
                else:
                    comp = obj.compare(existing)
                    # only update if newer
                    if comp != conduit.datatypes.COMPARISON_NEWER:
                        raise Exceptions.SynchronizeConflictError(comp, existing, obj)
                    else:
                        # overwrite and return new ID
                        uid = self._updateEvent(LUID, obj)
                        return uid

        # if we get here then it is new...
        log("Creating new object")
        uid = self._createEvent(obj)
        return uid
        
    def get_configuration(self):
        return {"username" : self.username,
                "password" : self.password,
                "selectedCalendarName" : self.selectedCalendarName, 
                "selectedCalendarURI" : self.selectedCalendarURI, }

    def get_UID(self):
        return self.username
		
